@@ -1,7 +1,20 @@
-setClass("SpeciesReference", representation(stoichiometry = "numeric", 
-  stoichiometryMath = "StoichiometryMath"), 
-  prototype = list(stoichiometry = 1, stoichiometryMath = NULL), 
-  contains = "SimpleSpeciesReference")
+setClassUnion("OptionalStoichiometryMath", c("StoichiometryMath", "NULL"))
+setClass("SpeciesReference",
+         representation(stoichiometry = "numeric",
+                        stoichiometryMath = "OptionalStoichiometryMath"), 
+         prototype(stoichiometry = 1), "SimpleSpeciesReference")
+
+setMethod("describe", "SpeciesReference", function(object) {
+  stoich <- stoichiometryMath(object)
+  if (is.null(stoich)) {
+    stoich <- stoichiometry(object)
+    if (stoich == 1)
+      stoich <- ""
+  } else stoich <- describe(stoich)
+  if (nchar(stoich))
+    stoich <- paste("(", stoich, ")", sep = "")
+  paste(stoich, callNextMethod(object), sep = "")
+})
 
  setGeneric("stoichiometry", function(object) standardGeneric("stoichiometry"))
 setMethod("stoichiometry", "SpeciesReference", function(object) object@stoichiometry)
@@ -17,13 +30,7 @@ setReplaceMethod("stoichiometry", "SpeciesReference", function(object, value) {
 
  setGeneric("stoichiometryMath<-", function(object, value) standardGeneric("stoichiometryMath<-"))
 setReplaceMethod("stoichiometryMath", "SpeciesReference", function(object, value) {
-  if (is.null(value))
-    new("SpeciesReference", notes = notes(object), annotation = annotation(object),
-      metaId = metaId(object), id = id(object), species = species(object), 
-      stoichiometry = stoichiometry(object))
-  else {
-    object@stoichiometryMath <- as.expression(value)
-    object
-  }
+  object@stoichiometryMath <- value
+  object
 })
 
