@@ -2,6 +2,16 @@ setClass("KineticLaw", representation(math = "expression", parameters = "list",
   timeUnits = "character", substanceUnits = "character"), 
   contains = "SBase")
 
+setMethod("describe", "KineticLaw",
+          function(object) {
+            desc <- as.character(math(object))
+            paramDescs <- paste(sapply(parameters(object), describe),
+                                collapse = ", ")
+            if (nchar(paramDescs))
+              desc <- paste(desc, "where", paramDescs)
+            desc
+          })
+
 setMethod("math", "KineticLaw", function(object) object@math)
 
  setGeneric("parameters", function(object) standardGeneric("parameters"))
@@ -13,7 +23,7 @@ setMethod("timeUnits", "KineticLaw", function(object) object@timeUnits)
 setMethod("substanceUnits", "KineticLaw", function(object) object@substanceUnits)
 
 setReplaceMethod("math", "KineticLaw", function(object, value) {
-  object@math <- as.expression(value)
+  object@math <- asMath(value)
   object
 })
 
@@ -21,6 +31,7 @@ setReplaceMethod("math", "KineticLaw", function(object, value) {
 setReplaceMethod("parameters", "KineticLaw", function(object, value) {
   if (!all(sapply(value, inherits, "Parameter")))
     stop("All parameters must be Parameter objects")
+  names(value) <- sapply(value, id)
   object@parameters <- value
   object
 })
