@@ -1,7 +1,10 @@
 setClass("ParameterRule", representation(name = "character", units = "character"),
   contains = "AssignmentRule")
 
- 
+setMethod("describe", "ParameterRule",
+          function(object) paste(name(object), ":=", math(object),
+                                 units(object)))
+
 setMethod("name", "ParameterRule", function(object) object@name)
 
 setMethod("units", "ParameterRule", function(x) x@units)
