@@ -1,13 +1,27 @@
 #include "rsbml.h"
 #include <R_ext/Parse.h>
+#include <stdlib.h>
 
-#define LIST_OF(var, class, func, field) \
+#define LIST_OF(var, class, func, field, id_attr) \
 ({ \
   int i, len = class ## _getNum ## field ## s(var); \
-  SEXP r_list; \
+  SEXP r_list, r_names; \
   PROTECT(r_list = NEW_LIST(len)); \
-  for (i = 0; i < len; i++) \
-    SET_VECTOR_ELT(r_list, i, rsbml_build_dom_ ## func(class ## _get ## field(var, i))); \
+  if (id_attr) \
+    PROTECT(r_names = NEW_CHARACTER(len)); \
+  for (i = 0; i < len; i++) { \
+    SEXP r_element = rsbml_build_dom_ ## func(class ## _get ## field(var, i)); \
+    SET_VECTOR_ELT(r_list, i, r_element); \
+    if (id_attr) { \
+      SEXP r_name = GET_SLOT(r_element, install(id_attr)); \
+      if (GET_LENGTH(r_name)) \
+        SET_STRING_ELT(r_names, i, STRING_ELT(r_name, 0)); \
+    } \
+  } \
+  if (id_attr) { \
+    SET_NAMES(r_list, r_names); \
+    UNPROTECT(1); \
+  } \
   UNPROTECT(1); \
   r_list; \
 })
@@ -15,38 +29,135 @@
 #include <Rversion.h>
 
 #if (R_VERSION >= R_Version(2,5,0))
-#define SRCFILE , NULL
+#define SRCFILE , R_NilValue
 #else
 #define SRCFILE
 #endif
 
-#define SET_MATH(class, var, Field, field) \
-({ \
-  ParseStatus status; \
-  char *math = SBML_formulaToString(class ## _get ## Field (var)); \
-  SET_SLOT(r_ ## var, install(#field), R_ParseVector(mkString(math), 1, &status \
-    SRCFILE)); \
-  if (status != PARSE_OK) { \
-    Rprintf("Failed to parse: %s\n", math); \
-    free(math); \
-	  error("Failed to parse mathematical formula in " #class); \
-  } \
-  free(math); \
+#define SET_MATH(class, var, Field, field) ({                      \
+  SEXP lang, expr;                                                 \
+  PROTECT(lang = rmathml_SEXP(class ## _get ## Field (var)));      \
+  expr = allocVector(EXPRSXP, 1);                                  \
+  SET_VECTOR_ELT(expr, 0, lang);                                   \
+  SET_SLOT(r_ ## var, install(#field), expr);                      \
+  UNPROTECT(1);                                                    \
 })
 
+#if LIBSBML_VERSION >= 50000
+#define Model_getNumLayouts(m) ({                                       \
+      LayoutModelPlugin_t *mplugin =                                    \
+        (LayoutModelPlugin_t *)(SBase_getPlugin((SBase_t *)m, "layout")); \
+      mplugin == NULL ? 0 : LayoutModelPlugin_getNumLayouts(mplugin);	\
+})
+#define Model_getLayout(m, i) ({                                        \
+      LayoutModelPlugin_t *mplugin =                                    \
+        (LayoutModelPlugin_t *)(SBase_getPlugin((SBase_t *)m, "layout")); \
+      LayoutModelPlugin_getLayout(mplugin, i);                          \
+})
+#else
 #define Model_getNumLayouts(m) \
-  ListOf_getNumItems(Model_getListOfLayouts(m));
+  ListOf_size(Model_getListOfLayouts(m));
+#endif
+
+
+static SEXP
+rsbml_build_dom_cvterm(CVTerm_t *cvterm)
+{
+  SEXP r_cvterm;
+  
+  PROTECT(r_cvterm = NEW_OBJECT(MAKE_CLASS("CVTerm")));
+  
+  const char *qualifier_type;
+  switch(CVTerm_getQualifierType(cvterm))
+  {
+    case MODEL_QUALIFIER:
+      qualifier_type = "model";
+    break;
+    case BIOLOGICAL_QUALIFIER:
+      qualifier_type = "biological";
+    break;
+    default:
+      qualifier_type = "unknown";
+  }
+  SET_SLOT(r_cvterm, install("qualifierType"), mkString(qualifier_type));
+  
+  switch(CVTerm_getModelQualifierType(cvterm))
+  {
+    case BQM_IS:
+      qualifier_type = "is";
+    break;
+    case BQM_IS_DESCRIBED_BY:
+      qualifier_type = "isDescribedBy";
+    break;
+    default:
+      qualifier_type = "unknown";
+  }
+  SET_SLOT(r_cvterm, install("modelQualifierType"), mkString(qualifier_type));
+  
+  switch(CVTerm_getBiologicalQualifierType(cvterm))
+  {
+    case BQB_IS:
+      qualifier_type = "is";
+    break;
+    case BQB_HAS_PART:
+      qualifier_type = "hasPart";
+    break;
+    case BQB_IS_PART_OF:
+      qualifier_type = "isPartOf";
+    break;
+    case BQB_IS_VERSION_OF:
+      qualifier_type = "isVersionOf";
+    break;
+    case BQB_HAS_VERSION:
+      qualifier_type = "hasVersion";
+    break;
+    case BQB_IS_HOMOLOG_TO:
+      qualifier_type = "isHomologTo";
+    break;
+    case BQB_IS_DESCRIBED_BY:
+      qualifier_type = "isDescribedBy";
+    break;
+#if LIBSBML_VERSION >= 30300
+  case BQB_IS_ENCODED_BY:
+    qualifier_type = "isEncodedBy";
+    break;
+  case BQB_ENCODES:
+    qualifier_type = "encodes";
+    break;
+  case BQB_OCCURS_IN:
+    qualifier_type = "occursIn";
+    break;
+#endif
+    default:
+      qualifier_type = "unknown";
+  }
+  SET_SLOT(r_cvterm, install("biologicalQualifierType"), mkString(qualifier_type));
+  
+  XMLAttributes_t *resources = CVTerm_getResources(cvterm);
+  SEXP r_resources;
+  PROTECT(r_resources = NEW_CHARACTER(XMLAttributes_getLength(resources)));
+  for (int i = 0; i < GET_LENGTH(r_resources); i++)
+    SET_STRING_ELT(r_resources, i, mkChar(XMLAttributes_getValue(resources, i)));
+  SET_SLOT(r_cvterm, install("resources"), r_resources);
+  
+  UNPROTECT(2);
+  
+  return r_cvterm;
+}
 
 static SEXP
 rsbml_build_dom_s_base(SEXP r_s_base, SBase_t *s_base)
 {
-  if (SBase_isSetNotes(s_base))
-    SET_SLOT(r_s_base, install("notes"), mkString(SBase_getNotes(s_base)));
-  if (SBase_isSetAnnotation(s_base))
-    SET_SLOT(r_s_base, install("annotation"), mkString(SBase_getAnnotation(s_base)));
   if (SBase_isSetMetaId(s_base))
     SET_SLOT(r_s_base, install("metaId"), mkString(SBase_getMetaId(s_base)));
-  
+  if (SBase_isSetNotes(s_base))
+    SET_SLOT(r_s_base, install("notes"), mkString(SBase_getNotesString(s_base)));
+  if (SBase_isSetAnnotation(s_base))
+    SET_SLOT(r_s_base, install("annotation"), mkString(SBase_getAnnotationString(s_base)));
+  if (SBase_isSetSBOTerm(s_base))
+    SET_SLOT(r_s_base, install("sboTerm"),
+             ScalarInteger(SBase_getSBOTerm(s_base)));
+  SET_SLOT(r_s_base, install("cvTerms"), LIST_OF(s_base, SBase, cvterm, CVTerm, NULL));
   return r_s_base;
 }
 
@@ -75,11 +186,11 @@ rsbml_build_dom_species(Species_t *species)
     SET_SLOT(r_species, install("spatialSizeUnits"), mkString(Species_getSpatialSizeUnits(species)));
   if (Species_isSetUnits(species))
     SET_SLOT(r_species, install("units"), mkString(Species_getUnits(species)));
-  SET_SLOT(r_species, install("hasOnlySubstanceUnits"), ScalarInteger(Species_getHasOnlySubstanceUnits(species)));
-  SET_SLOT(r_species, install("boundaryCondition"), ScalarInteger(Species_getBoundaryCondition(species)));
+  SET_SLOT(r_species, install("hasOnlySubstanceUnits"), ScalarLogical(Species_getHasOnlySubstanceUnits(species)));
+  SET_SLOT(r_species, install("boundaryCondition"), ScalarLogical(Species_getBoundaryCondition(species)));
   if (Species_isSetCharge(species))
     SET_SLOT(r_species, install("charge"), ScalarInteger(Species_getCharge(species)));
-  SET_SLOT(r_species, install("constant"), ScalarInteger(Species_getConstant(species)));
+  SET_SLOT(r_species, install("constant"), ScalarLogical(Species_getConstant(species)));
   
   UNPROTECT(1);
   return r_species;
@@ -102,7 +213,7 @@ rsbml_build_dom_parameter(Parameter_t *parameter)
     SET_SLOT(r_parameter, install("value"), ScalarReal(Parameter_getValue(parameter)));
   if (Parameter_isSetUnits(parameter))
     SET_SLOT(r_parameter, install("units"), mkString(Parameter_getUnits(parameter)));
-  SET_SLOT(r_parameter, install("constant"), ScalarInteger(Parameter_getConstant(parameter)));
+  SET_SLOT(r_parameter, install("constant"), ScalarLogical(Parameter_getConstant(parameter)));
   
   UNPROTECT(1);
   
@@ -128,7 +239,7 @@ rsbml_build_dom_kinetic_law(KineticLaw_t *kinetic_law)
     SET_SLOT(r_kinetic_law, install("substanceUnits"), mkString(KineticLaw_getSubstanceUnits(kinetic_law)));
   
   SET_SLOT(r_kinetic_law, install("parameters"), 
-    LIST_OF(kinetic_law, KineticLaw, parameter, Parameter));
+    LIST_OF(kinetic_law, KineticLaw, parameter, Parameter, "id"));
   
   UNPROTECT(1);
   return r_kinetic_law;
@@ -136,20 +247,32 @@ rsbml_build_dom_kinetic_law(KineticLaw_t *kinetic_law)
 
 static void
 rsbml_build_dom_simple_species_reference(SEXP r_simple_species_reference,
-  SimpleSpeciesReference_t *simple_species_reference)
+  SpeciesReference_t *simple_species_reference)
 { 
   rsbml_build_dom_s_base(r_simple_species_reference, (SBase_t *)simple_species_reference);
   
-  if (SimpleSpeciesReference_isSetSpecies(simple_species_reference))
+  if (SpeciesReference_isSetSpecies(simple_species_reference))
     SET_SLOT(r_simple_species_reference, install("species"), 
-      mkString(SimpleSpeciesReference_getSpecies(simple_species_reference)));
+      mkString(SpeciesReference_getSpecies(simple_species_reference)));
   #ifdef USE_LAYOUT
-  if (SimpleSpeciesReference_isSetId(simple_species_reference))
+  if (SpeciesReference_isSetId(simple_species_reference))
     SET_SLOT(r_simple_species_reference, install("id"), 
-      mkString(SimpleSpeciesReference_getId(simple_species_reference)));
+      mkString(SpeciesReference_getId(simple_species_reference)));
   #endif
 }
 
+static SEXP
+rsbml_build_dom_stoichiometry_math(const StoichiometryMath_t *stoichiometry_math)
+{
+  SEXP r_stoichiometry_math;
+  
+  PROTECT(r_stoichiometry_math = NEW_OBJECT(MAKE_CLASS("StoichiometryMath")));
+  
+  SET_MATH(StoichiometryMath, stoichiometry_math, Math, math);
+  
+  return r_stoichiometry_math;
+}
+
 static SEXP
 rsbml_build_dom_species_reference(SpeciesReference_t *species_reference)
 {
@@ -158,15 +281,11 @@ rsbml_build_dom_species_reference(SpeciesReference_t *species_reference)
   PROTECT(r_species_reference = NEW_OBJECT(MAKE_CLASS("SpeciesReference")));
   
   rsbml_build_dom_simple_species_reference(r_species_reference, 
-    (SimpleSpeciesReference_t *)species_reference);
+    (SpeciesReference_t *)species_reference);
   
   if (SpeciesReference_isSetStoichiometryMath(species_reference)) {
-    SEXP r_stoichiometry_math;
-    SpeciesReference_t *stoichiometry_math = species_reference;
-    PROTECT(r_stoichiometry_math = NEW_OBJECT(MAKE_CLASS("StoichiometryMath")));
-    SET_MATH(SpeciesReference, stoichiometry_math, StoichiometryMath, math);
-    SET_SLOT(r_species_reference, install("stoichiometryMath"), r_stoichiometry_math);
-    UNPROTECT(1);
+    SET_SLOT(r_species_reference, install("stoiciometryMath"),
+      rsbml_build_dom_stoichiometry_math(SpeciesReference_getStoichiometryMath(species_reference)));
   }
   
   SET_SLOT(r_species_reference, install("stoichiometry"), 
@@ -180,14 +299,14 @@ rsbml_build_dom_species_reference(SpeciesReference_t *species_reference)
 }
 
 static SEXP
-rsbml_build_dom_modifier_species_reference(ModifierSpeciesReference_t *modifier_species_reference)
+rsbml_build_dom_modifier_species_reference(SpeciesReference_t *modifier_species_reference)
 {
   SEXP r_modifier_species_reference;
   
   PROTECT(r_modifier_species_reference = NEW_OBJECT(MAKE_CLASS("ModifierSpeciesReference")));
   
   rsbml_build_dom_simple_species_reference(r_modifier_species_reference, 
-    (SimpleSpeciesReference_t *)modifier_species_reference);
+    (SpeciesReference_t *)modifier_species_reference);
   
   UNPROTECT(1);
   
@@ -209,13 +328,13 @@ rsbml_build_dom_reaction(Reaction_t *reaction)
     SET_SLOT(r_reaction, install("name"), mkString(Reaction_getName(reaction)));
   if (Reaction_isSetKineticLaw(reaction))
     SET_SLOT(r_reaction, install("kineticLaw"), rsbml_build_dom_kinetic_law(Reaction_getKineticLaw(reaction)));
-  SET_SLOT(r_reaction, install("reversible"), ScalarInteger(Reaction_getReversible(reaction)));
+  SET_SLOT(r_reaction, install("reversible"), ScalarLogical(Reaction_getReversible(reaction)));
   if (Reaction_isSetFast(reaction))
-    SET_SLOT(r_reaction, install("fast"), ScalarInteger(Reaction_getFast(reaction)));
+    SET_SLOT(r_reaction, install("fast"), ScalarLogical(Reaction_getFast(reaction)));
   
-  SET_SLOT(r_reaction, install("reactants"), LIST_OF(reaction, Reaction, species_reference, Reactant));
-  SET_SLOT(r_reaction, install("products"), LIST_OF(reaction, Reaction, species_reference, Product));
-  SET_SLOT(r_reaction, install("modifiers"), LIST_OF(reaction, Reaction, modifier_species_reference, Modifier));
+  SET_SLOT(r_reaction, install("reactants"), LIST_OF(reaction, Reaction, species_reference, Reactant, "species"));
+  SET_SLOT(r_reaction, install("products"), LIST_OF(reaction, Reaction, species_reference, Product, "species"));
+  SET_SLOT(r_reaction, install("modifiers"), LIST_OF(reaction, Reaction, modifier_species_reference, Modifier, "species"));
   
   UNPROTECT(1);
   return r_reaction;
@@ -244,7 +363,7 @@ rsbml_build_dom_compartment(Compartment_t *compartment)
     SET_SLOT(r_compartment, install("units"), mkString(Compartment_getUnits(compartment)));
   if (Compartment_isSetOutside(compartment))
     SET_SLOT(r_compartment, install("outside"), mkString(Compartment_getOutside(compartment)));
-  SET_SLOT(r_compartment, install("constant"), ScalarInteger(Compartment_getConstant(compartment)));
+  SET_SLOT(r_compartment, install("constant"), ScalarLogical(Compartment_getConstant(compartment)));
   
   UNPROTECT(1);
   
@@ -262,7 +381,7 @@ rsbml_build_dom_unit(Unit_t *unit)
   
   SET_SLOT(r_unit, install("kind"), mkString(UnitKind_toString(Unit_getKind(unit))));
   SET_SLOT(r_unit, install("exponent"), ScalarInteger(Unit_getExponent(unit)));
-  SET_SLOT(r_unit, install("scale"), ScalarInteger(Unit_getScale(unit)));
+  SET_SLOT(r_unit, install("unitScale"), ScalarInteger(Unit_getScale(unit)));
   SET_SLOT(r_unit, install("multiplier"), ScalarReal(Unit_getMultiplier(unit)));
   SET_SLOT(r_unit, install("offset"), ScalarReal(Unit_getOffset(unit)));
   
@@ -285,7 +404,7 @@ rsbml_build_dom_unit_definition(UnitDefinition_t *unit_definition)
   if (UnitDefinition_isSetName(unit_definition))
     SET_SLOT(r_unit_definition, install("name"), mkString(UnitDefinition_getName(unit_definition)));
   
-  SET_SLOT(r_unit_definition, install("units"), LIST_OF(unit_definition, UnitDefinition, unit, Unit));
+  SET_SLOT(r_unit_definition, install("units"), LIST_OF(unit_definition, UnitDefinition, unit, Unit, NULL));
   
   UNPROTECT(1);
   
@@ -325,31 +444,29 @@ rsbml_build_dom_rule(Rule_t *rule)
     break;
   case SBML_ASSIGNMENT_RULE:
     PROTECT(r_rule = NEW_OBJECT(MAKE_CLASS("AssignmentRule")));
-    if (AssignmentRule_isSetVariable((AssignmentRule_t *)rule))
-      SET_SLOT(r_rule, install("variable"), 
-        mkString(AssignmentRule_getVariable((AssignmentRule_t *)rule)));
+    if (Rule_isSetVariable((Rule_t *)rule))
+      SET_SLOT(r_rule, install("variable"),
+        mkString(Rule_getVariable(rule)));
     break;
   case SBML_RATE_RULE:
     PROTECT(r_rule = NEW_OBJECT(MAKE_CLASS("RateRule")));
-    SET_SLOT(r_rule, install("variable"), mkString(RateRule_getVariable((RateRule_t *)rule)));
+    SET_SLOT(r_rule, install("variable"), mkString(
+      Rule_getVariable(rule)));
     break;
   case SBML_SPECIES_CONCENTRATION_RULE:
     PROTECT(r_rule = NEW_OBJECT(MAKE_CLASS("SpeciesConcentrationRule")));
-    SET_SLOT(r_rule, install("species"), 
-      mkString(SpeciesConcentrationRule_getSpecies((SpeciesConcentrationRule_t *)rule)));
+    SET_SLOT(r_rule, install("species"), mkString(
+      Rule_getVariable(rule)));
     break;
   case SBML_COMPARTMENT_VOLUME_RULE:
     PROTECT(r_rule = NEW_OBJECT(MAKE_CLASS("CompartmentVolumeRule")));
-    SET_SLOT(r_rule, install("compartment"), 
-      mkString(CompartmentVolumeRule_getCompartment((CompartmentVolumeRule_t *)rule)));
+    SET_SLOT(r_rule, install("compartment"), mkString(
+      Rule_getVariable(rule)));
     break;
   case SBML_PARAMETER_RULE:
     PROTECT(r_rule = NEW_OBJECT(MAKE_CLASS("ParameterRule")));
-    SET_SLOT(r_rule, install("name"), 
-      mkString(ParameterRule_getName((ParameterRule_t *)rule)));
-    if (ParameterRule_isSetUnits((ParameterRule_t *)rule))
-      SET_SLOT(r_rule, install("units"), 
-        mkString(ParameterRule_getUnits((ParameterRule_t *)rule)));
+    if (Rule_isSetUnits(rule))
+      SET_SLOT(r_rule, install("units"), mkString(Rule_getUnits(rule)));
     break;
   default:
     error("unknown rule type");
@@ -357,14 +474,17 @@ rsbml_build_dom_rule(Rule_t *rule)
   
   if (type == SBML_SPECIES_CONCENTRATION_RULE || type == SBML_COMPARTMENT_VOLUME_RULE ||
       type == SBML_PARAMETER_RULE) {
-    SET_SLOT(r_rule, install("type"), 
-      mkString(RuleType_toString(AssignmentRule_getType((AssignmentRule_t *)rule))));
+    RuleType_t type = Rule_getType(rule);
+    const char *type_name = "invalid";
+    if (type == RULE_TYPE_RATE)
+      type_name = "rate";
+    else if (type == RULE_TYPE_SCALAR)
+      type_name = "scalar";
+    SET_SLOT(r_rule, install("type"), mkString(type_name));
   }
   
   rsbml_build_dom_s_base(r_rule, (SBase_t *)rule);
   
-  if (Rule_isSetFormula(rule))
-    SET_SLOT(r_rule, install("formula"), mkString(Rule_getFormula(rule)));
   if (Rule_isSetMath(rule))
     SET_MATH(Rule, rule, Math, math);
   
@@ -393,6 +513,37 @@ rsbml_build_dom_event_assignment(EventAssignment_t *event_assignment)
   return r_event_assignment;
 }
 
+static SEXP
+rsbml_build_dom_delay(const Delay_t *delay)
+{
+  SEXP r_delay;
+  
+  PROTECT(r_delay = NEW_OBJECT(MAKE_CLASS("Delay")));
+  
+  rsbml_build_dom_s_base(r_delay, (SBase_t *)delay);
+  
+  SET_MATH(Delay, delay, Math, math);
+  
+  UNPROTECT(1);
+  
+  return r_delay;
+}
+static SEXP
+rsbml_build_dom_trigger(const Trigger_t *trigger)
+{
+  SEXP r_trigger;
+  
+  PROTECT(r_trigger = NEW_OBJECT(MAKE_CLASS("Trigger")));
+  
+  rsbml_build_dom_s_base(r_trigger, (SBase_t *)trigger);
+  
+  SET_MATH(Trigger, trigger, Math, math);
+  
+  UNPROTECT(1);
+  
+  return r_trigger;
+}
+
 static SEXP
 rsbml_build_dom_event(Event_t *event)
 {
@@ -406,15 +557,18 @@ rsbml_build_dom_event(Event_t *event)
     SET_SLOT(r_event, install("id"), mkString(Event_getId(event)));
   if (Event_isSetName(event))
     SET_SLOT(r_event, install("name"), mkString(Event_getName(event)));
-  if (Event_isSetTrigger(event))
-    SET_MATH(Event, event, Trigger, trigger);
   if (Event_isSetDelay(event))
-    SET_MATH(Event, event, Delay, delay);
+    SET_SLOT(r_event, install("eventDelay"), rsbml_build_dom_delay(Event_getDelay(event)));
+  if (Event_isSetTrigger(event))
+    SET_SLOT(r_event, install("trigger"), rsbml_build_dom_trigger(Event_getTrigger(event)));
+#if LIBSBML_VERSION >= 30300
+  // TODO: handle the useValueFromTrigger field
+#endif
   if (Event_isSetTimeUnits(event))
     SET_SLOT(r_event, install("timeUnits"), mkString(Event_getTimeUnits(event)));
   
   SET_SLOT(r_event, install("eventAssignments"), 
-    LIST_OF(event, Event, event_assignment, EventAssignment));
+    LIST_OF(event, Event, event_assignment, EventAssignment, NULL));
 
   UNPROTECT(1);
   
@@ -562,7 +716,7 @@ rsbml_build_dom_curve(Curve_t *curve)
   rsbml_build_dom_s_base(r_curve, (SBase_t *)curve);
   
   SET_SLOT(r_curve, install("curveSegments"), 
-    LIST_OF(curve, Curve, line_segment, CurveSegment));
+    LIST_OF(curve, Curve, line_segment, CurveSegment, NULL));
   
   UNPROTECT(1);
   
@@ -587,10 +741,9 @@ rsbml_build_dom_species_reference_glyph(SpeciesReferenceGlyph_t *species_referen
   if (SpeciesReferenceGlyph_isSetRole(species_reference_glyph))
     SET_SLOT(r_species_reference_glyph, install("role"), 
       mkString(SpeciesReferenceGlyph_getRoleString(species_reference_glyph)));
-    // FIXME: SpeciesReferenceGlyph_getCurve is not exported
-  /*if (SpeciesReferenceGlyph_isSetCurve(species_reference_glyph))  
+  if (SpeciesReferenceGlyph_isSetCurve(species_reference_glyph))  
     SET_SLOT(r_species_reference_glyph, install("curve"), 
-      rsbml_build_dom_curve(SpeciesReferenceGlyph_getCurve(species_reference_glyph)));*/
+      rsbml_build_dom_curve(SpeciesReferenceGlyph_getCurve(species_reference_glyph)));
   
   UNPROTECT(1);
   
@@ -607,12 +760,11 @@ rsbml_build_dom_reaction_glyph(ReactionGlyph_t *reaction_glyph)
   rsbml_build_dom_base_graphical_object(r_reaction_glyph, (GraphicalObject_t *)reaction_glyph);
   
   SET_SLOT(r_reaction_glyph, install("reaction"), mkString(ReactionGlyph_getReactionId(reaction_glyph)));
-  // FIXME: ReactionGlyph_getCurve is not exported for some reason
-  /*if (ReactionGlyph_isSetCurve(reaction_glyph))
+  if (ReactionGlyph_isSetCurve(reaction_glyph))
     SET_SLOT(r_reaction_glyph, install("curve"), 
-      rsbml_build_dom_curve(ReactionGlyph_getCurve(reaction_glyph)));*/
+      rsbml_build_dom_curve(ReactionGlyph_getCurve(reaction_glyph)));
   SET_SLOT(r_reaction_glyph, install("speciesReferenceGlyphs"), 
-    LIST_OF(reaction_glyph, ReactionGlyph, species_reference_glyph, SpeciesReferenceGlyph));
+    LIST_OF(reaction_glyph, ReactionGlyph, species_reference_glyph, SpeciesReferenceGlyph, "id"));
   
   UNPROTECT(1);
   
@@ -664,19 +816,18 @@ rsbml_build_dom_layout(Layout_t *layout)
   rsbml_build_dom_s_base(r_layout, (SBase_t *)layout);
   
   SET_SLOT(r_layout, install("id"), mkString(Layout_getId(layout)));
-  // FIXME: libsbml does not provide Layout_getDimensions()
-  //SET_SLOT(r_layout, install("dimensions"), rsbml_build_dom_dimensions(Layout_getDimensions(layout)));
+  SET_SLOT(r_layout, install("dimensions"), rsbml_build_dom_dimensions(Layout_getDimensions(layout)));
   
   SET_SLOT(r_layout, install("compartmentGlyphs"), 
-    LIST_OF(layout, Layout, compartment_glyph, CompartmentGlyph));
+    LIST_OF(layout, Layout, compartment_glyph, CompartmentGlyph, "id"));
   SET_SLOT(r_layout, install("speciesGlyphs"), 
-    LIST_OF(layout, Layout, species_glyph, SpeciesGlyph));
+    LIST_OF(layout, Layout, species_glyph, SpeciesGlyph, "id"));
   SET_SLOT(r_layout, install("reactionGlyphs"), 
-    LIST_OF(layout, Layout, reaction_glyph, ReactionGlyph));
+    LIST_OF(layout, Layout, reaction_glyph, ReactionGlyph, "id"));
   SET_SLOT(r_layout, install("textGlyphs"), 
-    LIST_OF(layout, Layout, text_glyph, TextGlyph));
+    LIST_OF(layout, Layout, text_glyph, TextGlyph, "id"));
   SET_SLOT(r_layout, install("additionalGraphicalObjects"), 
-    LIST_OF(layout, Layout, graphical_object, AdditionalGraphicalObject));
+    LIST_OF(layout, Layout, graphical_object, AdditionalGraphicalObject, "id"));
 
   UNPROTECT(1);
   
@@ -684,42 +835,162 @@ rsbml_build_dom_layout(Layout_t *layout)
 }
 #endif /* USE_LAYOUT */
 
+static SEXP
+rsbml_build_dom_compartment_type(CompartmentType_t *compartment_type)
+{
+  SEXP r_compartment_type;
+  
+  PROTECT(r_compartment_type = NEW_OBJECT(MAKE_CLASS("CompartmentType")));
+  
+  rsbml_build_dom_s_base(r_compartment_type, (SBase_t *)compartment_type);
+  
+  SET_SLOT(r_compartment_type, install("id"), mkString(CompartmentType_getId(compartment_type)));
+  if (CompartmentType_isSetName(compartment_type))
+    SET_SLOT(r_compartment_type, install("name"), mkString(CompartmentType_getName(compartment_type)));
+  
+  UNPROTECT(1);
+  
+  return r_compartment_type;
+}
+static SEXP
+rsbml_build_dom_species_type(SpeciesType_t *species_type)
+{
+  SEXP r_species_type;
+  
+  PROTECT(r_species_type = NEW_OBJECT(MAKE_CLASS("SpeciesType")));
+  
+  rsbml_build_dom_s_base(r_species_type, (SBase_t *)species_type);
+  
+  SET_SLOT(r_species_type, install("id"), mkString(SpeciesType_getId(species_type)));
+  if (SpeciesType_isSetName(species_type))
+    SET_SLOT(r_species_type, install("name"), mkString(SpeciesType_getName(species_type)));
+  
+  UNPROTECT(1);
+  
+  return r_species_type;
+}
+static SEXP
+rsbml_build_dom_initial_assignment(InitialAssignment_t *initial_assignment)
+{
+  SEXP r_initial_assignment;
+  
+  PROTECT(r_initial_assignment = NEW_OBJECT(MAKE_CLASS("InitialAssignment")));
+  
+  rsbml_build_dom_s_base(r_initial_assignment, (SBase_t *)initial_assignment);
+  
+  SET_SLOT(r_initial_assignment, install("symbol"), mkString(InitialAssignment_getSymbol(initial_assignment)));
+  SET_MATH(InitialAssignment, initial_assignment, Math, math);
+  
+  UNPROTECT(1);
+  
+  return r_initial_assignment;
+}
+static SEXP
+rsbml_build_dom_constraint(Constraint_t *constraint)
+{
+  SEXP r_constraint;
+  
+  PROTECT(r_constraint = NEW_OBJECT(MAKE_CLASS("Constraint")));
+  
+  rsbml_build_dom_s_base(r_constraint, (SBase_t *)constraint);
+  
+  SET_SLOT(r_constraint, install("message"), 
+    mkString(XMLNode_convertXMLNodeToString(Constraint_getMessage(constraint))));
+  SET_MATH(Constraint, constraint, Math, math);
+  
+  UNPROTECT(1);
+  
+  return r_constraint;
+}
+
+static SEXP
+rsbml_build_dom_model_creator(ModelCreator_t *model_creator)
+{
+  SEXP r_model_creator;
+  
+  PROTECT(r_model_creator = NEW_OBJECT(MAKE_CLASS("ModelCreator")));
+  
+  SET_SLOT(r_model_creator, install("familyName"), mkString(ModelCreator_getFamilyName(model_creator)));
+  SET_SLOT(r_model_creator, install("givenName"), mkString(ModelCreator_getGivenName(model_creator)));
+  SET_SLOT(r_model_creator, install("email"), mkString(ModelCreator_getEmail(model_creator)));
+  SET_SLOT(r_model_creator, install("organization"), mkString(ModelCreator_getOrganisation(model_creator)));
+  
+  UNPROTECT(1);
+  
+  return r_model_creator;
+}
+
+static SEXP
+rsbml_build_dom_model_history(ModelHistory_t *model_history)
+{
+  SEXP r_model_history;
+  
+  PROTECT(r_model_history = NEW_OBJECT(MAKE_CLASS("ModelHistory")));
+
+  if (ModelHistory_getCreatedDate(model_history))
+    SET_SLOT(r_model_history, install("createdDate"), 
+             mkString(Date_getDateAsString(ModelHistory_getCreatedDate(model_history))));
+  if (ModelHistory_getModifiedDate(model_history))
+    SET_SLOT(r_model_history, install("modifiedDate"), 
+             mkString(Date_getDateAsString(ModelHistory_getModifiedDate(model_history))));
+  SET_SLOT(r_model_history, install("creators"), 
+    LIST_OF(model_history, ModelHistory, model_creator, Creator, NULL));
+  
+  UNPROTECT(1);
+  
+  return r_model_history;
+}
+
 static SEXP
 rsbml_build_dom_model(Model_t *model)
 {
   SEXP r_model;
-  
+
   PROTECT(r_model = NEW_OBJECT(MAKE_CLASS("Model")));
+
+  rsbml_build_dom_s_base(r_model, (SBase_t *)model);
   
   if (Model_isSetId(model))
     SET_SLOT(r_model, install("id"), mkString(Model_getId(model)));
   if (Model_isSetName(model))
     SET_SLOT(r_model, install("name"), mkString(Model_getName(model)));
-  
+
+  if (Model_isSetModelHistory(model))
+    SET_SLOT(r_model, install("modelHistory"), 
+      rsbml_build_dom_model_history(Model_getModelHistory(model)));
+
   { /* species don't fit into the macro */
-    /* on the bright side, this is an example of the macro */
     int i, num_species = Model_getNumSpecies(model);
-    SEXP r_species;
+    SEXP r_species, r_names;
     PROTECT(r_species = NEW_LIST(num_species));
-    for (i = 0; i < num_species; i++)
-      SET_VECTOR_ELT(r_species, i, rsbml_build_dom_species(Model_getSpecies(model, i)));
+    PROTECT(r_names = NEW_CHARACTER(num_species));
+    for (i = 0; i < num_species; i++) {
+      SEXP r_element = rsbml_build_dom_species(Model_getSpecies(model, i));
+      SET_VECTOR_ELT(r_species, i, r_element);
+      SET_STRING_ELT(r_names, i, STRING_ELT(GET_SLOT(r_element, install("id")),0));
+    }
+    SET_NAMES(r_species, r_names);
     SET_SLOT(r_model, install("species"), r_species);
-    UNPROTECT(1);
+    UNPROTECT(2);
   }
-  
-  SET_SLOT(r_model, install("reactions"), LIST_OF(model, Model, reaction, Reaction));
-  SET_SLOT(r_model, install("compartments"), LIST_OF(model, Model, compartment, Compartment));
-  SET_SLOT(r_model, install("unitDefinitions"), LIST_OF(model, Model, unit_definition, UnitDefinition));
-  SET_SLOT(r_model, install("functionDefinitions"), LIST_OF(model, Model, function_definition, FunctionDefinition));
-  SET_SLOT(r_model, install("parameters"), LIST_OF(model, Model, parameter, Parameter));
-  SET_SLOT(r_model, install("rules"), LIST_OF(model, Model, rule, Rule));
-  SET_SLOT(r_model, install("events"), LIST_OF(model, Model, event, Event));
+
+  SET_SLOT(r_model, install("reactions"), LIST_OF(model, Model, reaction, Reaction, "id"));
+  SET_SLOT(r_model, install("compartments"), LIST_OF(model, Model, compartment, Compartment, "id"));
+  SET_SLOT(r_model, install("unitDefinitions"), LIST_OF(model, Model, unit_definition, UnitDefinition, "id"));
+  SET_SLOT(r_model, install("functionDefinitions"), LIST_OF(model, Model, function_definition, FunctionDefinition, "id"));
+  SET_SLOT(r_model, install("parameters"), LIST_OF(model, Model, parameter, Parameter, "id"));
+  SET_SLOT(r_model, install("rules"), LIST_OF(model, Model, rule, Rule, NULL));
+  SET_SLOT(r_model, install("events"), LIST_OF(model, Model, event, Event, "id"));
+  SET_SLOT(r_model, install("speciesTypes"), LIST_OF(model, Model, species_type, SpeciesType, "id"));
+  SET_SLOT(r_model, install("compartmentTypes"), LIST_OF(model, Model, compartment_type, CompartmentType, "id"));
+  SET_SLOT(r_model, install("initialAssignments"), LIST_OF(model, Model, initial_assignment, InitialAssignment, NULL));
+  SET_SLOT(r_model, install("constraints"), LIST_OF(model, Model, constraint, Constraint, "id"));
   #ifdef USE_LAYOUT
-  SET_SLOT(r_model, install("layouts"), LIST_OF(model, Model, layout, Layout));
+  SET_SLOT(r_model, install("layouts"), LIST_OF(model, Model, layout, Layout, "id"));
   #endif
 
   UNPROTECT(1);
-  
+
   return r_model;
 }
 
@@ -731,8 +1002,16 @@ rsbml_build_dom(SBMLDocument_t *doc)
   PROTECT(r_dom = NEW_OBJECT(MAKE_CLASS("SBML")));
   
   SET_SLOT(r_dom, install("level"), ScalarInteger(SBMLDocument_getLevel(doc)));
-  SET_SLOT(r_dom, install("version"), ScalarInteger(SBMLDocument_getVersion(doc)));
+  SET_SLOT(r_dom, install("ver"), ScalarInteger(SBMLDocument_getVersion(doc)));
   SET_SLOT(r_dom, install("model"), rsbml_build_dom_model(SBMLDocument_getModel(doc)));
+
+#if LIBSBML_VERSION >= 50000
+#ifdef USE_LAYOUT
+  SBase_enablePackage((SBase_t *)doc,
+                      "http://projects.eml.org/bcb/sbml/level2", "layout",
+                      TRUE);
+#endif
+#endif
   
   UNPROTECT(1);
   
@@ -743,7 +1022,7 @@ SEXP
 rsbml_R_build_dom(SEXP r_doc)
 {
   SBMLDocument_t *doc = R_ExternalPtrAddr(r_doc);
-  if (SBMLDocument_getNumErrors(doc) || SBMLDocument_getNumFatals(doc))
+  if (rsbml_errors(doc))
     error("Cannot build dom from document with errors");
   return rsbml_build_dom(doc);
 }
